package com.persagy.dmp.rwd.digital.handler;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.amqp.handler.AbstractRabbitHandlerTemplate;
import com.persagy.dmp.basic.model.DigitalManageMessage;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.dmp.file.model.UploadMes;
import com.persagy.dmp.rwd.basic.constant.DigitalMessageConstant;
import com.persagy.dmp.rwd.digital.service.impl.ObjectDigitalServiceImpl;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.DataInput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Description
 *
 * @author:linhuili
 * @date:2021/11/8
 */
@Slf4j
@Component
@RabbitListener(queues = "queue-dmp-rwd")
public class DigitalObjectMessageHandler extends AbstractRabbitHandlerTemplate {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private ObjectDigitalServiceImpl objectDigitalService;

    @Override
    protected void handler(String message, Channel channel, Message vo) {
          try {
            log.info("接收对象消息：{}", message);
            //接收对象消息
            DigitalManageMessage messageVO = objectMapper.readValue(message, DigitalManageMessage.class);
            ObjectNode objectDigital = messageVO.getNewObj();
            if(objectDigital == null || StringUtils.isEmpty(messageVO.getOperatorType())){
                log.info("新增消息格式有误：{}"+ messageVO);
            }
            ObjectDigital objectDigitalVO = objectMapper.readValue((DataInput) objectDigital, ObjectDigital.class);
            //创建二维码图片
            if(DigitalMessageConstant.OPERATE_AFTER_INSERT.equals(messageVO.getOperatorType())){
                createObjectQrCode(objectDigitalVO);
            }
            //删除二维码信息
            if(DigitalMessageConstant.OPERATE_AFTER_DELETE.equals(messageVO.getOperatorType())){
                removeObjectQrCode(objectDigitalVO);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new BusinessException(e.getMessage(), e);
        }
    }

    /**
     * 生成二维码信息
     * @param digital
     */
    private void createObjectQrCode(ObjectDigital digital){
        //生成二维码图片并上传到文件服务器
        UploadMes objectQrCode = objectDigitalService.createObjectQrCode(digital);
        if(objectQrCode == null){
            log.info("生成二维码失败");
            return;
        }
        //设置对象二维码信息
        objectDigitalService.setObjectQrCode(digital,objectQrCode);
        //更新对象二维码信息
        List<ObjectDigital> list = new ArrayList<>();
        list.add(digital);
        objectDigitalService.update(list);
    }

    /**
     * 移除对象二维码信息点
     * @param digital
     */
    public void removeObjectQrCode(ObjectDigital digital){
        ObjectNode infos = JsonNodeFactory.instance.objectNode();
        //删除二维码信息点
        ArrayNode removeInfos = JsonNodeFactory.instance.arrayNode();
        removeInfos.add(CommonConstant.QR_CODE_PIC_INFO);
        infos.put(CommonConstant.REMOVE_FIELD, removeInfos);
        digital.setInfos(infos);

        List<ObjectDigital> list = new ArrayList<>();
        list.add(digital);
        objectDigitalService.update(list);
    }
}
